/*******************************************************************************
* Copyright (C) 2014 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** \brief This example shows how to  measure a voltage with the SK-FM4-216-
 ** ETHERNET ADC and display on the LCD via I2C bus
 **
 ** This example samples the potentiometer voltage on channel 14 of the
 ** ADC, displays the voltage value on LCD display via I2C bus
 ** and prints it via UART0 to a terminal program.
 **
 ** History:
 **   - 2014-09-18  1.0  Deepak Naik        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "i2clcd.h"
#include <stdio.h>                                  // needed for sprintf()

/******************************************************************************/
/* Global variable definitions                                                */
/******************************************************************************/
uint16_t u16AdcValue;

/**
 ******************************************************************************
 ** \brief   Wait routine (delay)
 **
 ** \param  [in] u32WaitTime  Wait delay
 ******************************************************************************/
static void Wait(volatile uint32_t u32WaitTime)
{
  while (u32WaitTime--)
  {}
}

/**
 ******************************************************************************
 ** \brief   Initialization of MFS0 in UART-Mode
 ******************************************************************************/
static void InitMfs0(void)
{
  // Initialize ports for MFS0
  FM4_GPIO->PFR2   = 0x06u;                         // P21>SIN0_0, P22>SOT0_0
  FM4_GPIO->EPFR07 &= 0xFFFFFF0Ful;
  FM4_GPIO->EPFR07 |= 0x00000040ul;
  FM4_GPIO->ADE &= ~(1ul << 31ul);                  // switch off AN31 for SOT0_0

  // Initialize MFS to UART asynchronous mode
  FM4_MFS0->SMR =  0x01u;                           // serial data output enable, operation mode 0
  FM4_MFS0->SCR =  0x80u;                           // MFS reset
  FM4_MFS0->SSR =  0x80u;                           // 0x80 = clear receive error flag
  FM4_MFS0->ESCR = 0x00u;

  FM4_MFS0->BGR = 100000000ul / 115200ul - 1ul;       // 115.2kbaud @ 100 MHz

  FM4_MFS0->SCR |= 0x03u;                           // RX, TX enable
}

/**
 ******************************************************************************
 ** \brief   Send character
 **
 ** \param   [in] u8Char   Character to be sent
 ******************************************************************************/
void PutCharMfs0(uint8_t u8Char)
{
  while (!(FM4_MFS0->SSR_f.TDRE));                  // wait for transmit buffer empty
  FM4_MFS0->TDR = u8Char;                           // put u8Char into buffer

  if ('\n' == u8Char)
  {
    while (!(FM4_MFS0->SSR_f.TDRE));                // wait for transmit buffer empty
    FM4_MFS0->TDR = '\r';                           // put '\r' into buffer
  }
}

/**
 ******************************************************************************
 ** \brief    Send a complete string (0-terminated)
 **
 ** \param    [in] pu8Buffer   Pointer to (constant) file of bytes in mem
 ******************************************************************************/
void PutStringMfs0(uint8_t *pu8Buffer)
{
  while (*pu8Buffer != '\0')
  {
    PutCharMfs0(*pu8Buffer++);                      // send every char of string
  }
}

/**
 ******************************************************************************
 ** \brief   Initialization of ADC0
 ******************************************************************************/
static void InitAdc0(void)
{
  FM4_GPIO->ADE_f.AN14 = 1u;                        // Select analog pin -> AN14

  FM4_ADC0->SCIS3 = 0x00u;                          // Scan channel select AN31-AN24
  FM4_ADC0->SCIS2 = 0x00u;                          // Scan channel select AN23-AN16
  FM4_ADC0->SCIS1 = 0x40u;                          // Scan channel select AN15-AN08 -> AN14
  FM4_ADC0->SCIS0 = 0x00u;                          // Scan channel select AN07-AN00

  FM4_ADC0->ADST1 = 0x2Fu;                          // Sampling Time 1
  FM4_ADC0->ADST0 = 0x2Fu;                          // Sampling Time 0

  FM4_ADC0->ADSS3 = 0x00u;                          // Sampling Time Select AN31-AN24
  FM4_ADC0->ADSS2 = 0x00u;                          // Sampling Time Select AN23-AN16
  FM4_ADC0->ADSS1 = 0x00u;                          // Sampling Time Select AN15-AN08 -> Use Sampling Time 0 for AN14
  FM4_ADC0->ADSS0 = 0x00u;                          // Sampling Time Select AN07-AN00

  FM4_ADC0->ADCT  = 0x00u;                          // Comparison Time = 14 x (ADCT + 2) / HCLK

  FM4_ADC0->ADCEN = 0x01u;                          // Enable ADC

  while (3u != FM4_ADC0->ADCEN);                    // wait until ADC operation is enabled

  FM4_ADC0->CMPCR = 0x00u;                          // No comparison
  FM4_ADC0->CMPD  = 0x00u;                          // No comparison value

  FM4_ADC0->SFNS  = 0x00u;                          // Set Fifo Stage Count Interrupt
  FM4_ADC0->ADCR  = 0x00u;                          // Disable ADC interrupts
  FM4_ADC0->ADSR  = 0x40u;                          // Result on LSB side

  FM4_ADC0->SCCR = 0x11u;                           // FIFO clear, start ADC Single Conversion
}

/**
 ******************************************************************************
 ** \brief   Get sample data from ADC0
 **
 ** \return  Sample value
 ******************************************************************************/
static uint16_t GetAdc0Value(void)
{
  uint16_t u16AdcValue;

  while (0u != ((FM4_ADC0->SCFDL) & 0x1000u))       // Is valid data available?
  {}

  u16AdcValue = FM4_ADC0->SCFDH;

  return u16AdcValue;
}



/**
 ******************************************************************************
 ** \brief  Main function of project for S6E2CC series.
 **
 ** \return uint32_t  return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  float   f16AdcValue;
  uint8_t au8String[20u];

  InitMfs0();
  InitAdc0();

  I2c_Init();
  I2cLcd_Init();
  I2cLcd_SetBG(0x2Fu);
    I2cLcd_Write(1u, "Voltage is: ");

  while(1u)
  {
    bFM4_ADC0_SCCR_SSTR = 1u;                       // Start ADC Single Conversion
    u16AdcValue = GetAdc0Value();

    f16AdcValue = (float)5u * ((float)u16AdcValue / (float)4096u);
    sprintf((char*)au8String, "%f V\n", f16AdcValue);

    PutStringMfs0(au8String);                       // Send the voltage value to UART0

    I2cLcd_Write(2u, (char*)au8String);

    Wait(100000ul);
  }
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
